@@ -8,7 +8,7 @@
 # 特記事項:
 # なし
 #
-# Copyright (c) 2012-2019, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
+# Copyright (c) 2012-2015, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
 #
 #####################################################################
 package PGRex;
@@ -18,8 +18,6 @@ use strict;
 use Net::OpenSSH;
 use PGRex::command;
 use Class::Struct;
-use Fcntl;
-use Errno;
 
 BEGIN {
     if ($ENV{'LANG'} =~ m/ja/i){
@@ -46,7 +44,7 @@ struct Pg_dir_state => {
 };
 
 use constant {
-    VERSIONNUM       => "11.0",
+    VERSIONNUM       => "1.7.2",
     VERSIONINFO      => "[0] (pg-rex_operation_tools) [1]\n",
     CONFIG_PATH      => "/etc/",
     CONFIG_FILENAME  => "pg-rex_tools.conf",
@@ -54,14 +52,11 @@ use constant {
     CIB_FILENAME     => "cib.xml",
     HACF_PATH        => "/etc/corosync/",
     HACF_FILENAME    => "corosync.conf",
+    AUTHKEY_FILENAME => "authkey",
     RA_TMPDIR        => "/var/lib/pgsql/tmp",
-    LOCK_FILENAME    => "PGSQL.lock",
-    PID_FILENAME     => "pg-rex_tools.pid",
-    PID_FILEDIR      => "/var/run"
+    LOCK_FILENAME    => "PGSQL.lock"
 };
 
-our $additional_information_mode = 0;
-
 sub pacemaker_running{
     my ($ssh_info) = @_;
     my $result;
@@ -177,10 +172,8 @@ sub network_running{
         
         # crm_attribute の実行結果が "<IPアドレス> is UP"であれば、corosync.conf の設定と
         # IPアドレスが一致しているかを確認する
-        # なお、crm_attribute の実行結果から取得できるIPアドレスのため、
-        # 厳密なIPv4の正規表現でIPアドレスを取得しない
         if ($result =~ /^(\d+\.\d+\.\d+\.\d+)\sis\sUP/){
-            @matched = grep {$_ eq $1} @{$$iclan_ipaddr{'my_node'}};
+            @matched = grep {/$1/} @{$$iclan_ipaddr{'my_node'}};
             if (@matched == 0){
                 return 0;
             }
@@ -200,10 +193,8 @@ sub network_running{
         
         # crm_attribute の実行結果が "<IPアドレス> is UP"であれば、corosync.conf の設定と
         # IPアドレスが一致しているかを確認する
-        # なお、crm_attribute の実行結果から取得できるIPアドレスのため、
-        # 厳密なIPv4の正規表現でIPアドレスを取得しない
         if ($result =~ /^(\d+\.\d+\.\d+\.\d+)\sis\sUP/){
-            @matched = grep {$_ eq $1} @{$$iclan_ipaddr{'another_node'}};
+            @matched = grep {/$1/} @{$$iclan_ipaddr{'another_node'}};
             if (@matched == 0){
                 return 0;
             }
@@ -446,13 +437,14 @@ sub diskd_running{
 
 
 sub read_cib{
-    my ($cib_path, $pg_primitive_resource_id, $kill_when_no_data) = @_;
+    my ($cib_path, $pg_primitive_resource_id, $kill_when_no_data, $config_path, $dbcluster_dir) = @_;
     my @cib_strings;
     my %cib_value;
     my $my_node;
     my $my_node_id;
     my $my_pgsql_data_status_id;
     my @check_key_list = ("pgdata", "repuser");
+    my $config_comment = "# この設定値はツールが自動で追加したものです。設定行が複数行出る場合がありますが動作に影響はありません。";
     my $result;
     my $param_name;
     my $param_value;
@@ -461,7 +453,7 @@ sub read_cib{
     chomp $my_node;
 
     # tmpdir のデフォルト値を設定する
-    $cib_value{'tmpdir'} = RA_TMPDIR;
+    $cib_value{'tmpdir'} = "/var/lib/pgsql/tmp";
 
     open (FILE, $cib_path) or printlog("ERROR", COMMON_MS0023, $cib_path);
     @cib_strings = <FILE>;
@@ -505,6 +497,11 @@ sub read_cib{
         }
     }
 
+    # pg-rex_tools.conf にDBクラスタパスの記述が無い場合追加する
+    if (! defined ($dbcluster_dir)){
+        exec_command("$ECHO \"ARCHIVE_DELETE_DBCLUSTER_DIR = $cib_value{'pgdata'}  $config_comment\" >> $config_path");
+    
+    }
     return %cib_value;
 }
 
@@ -521,6 +518,7 @@ sub read_config{
     $config_value{'VIP_SLAVE'} = "enable";
     $config_value{'PEER_NODE_SSH_PASS_MODE'} = "manual";
     $config_value{'BACKUP_NODE_SSH_PASS_MODE'} = "manual";
+    $config_value{'PG_BASEBACKUP_opt'} = "-X stream";
     
     open (FILE, $config_path) or printlog("ERROR", COMMON_MS0024, $config_path);
     @config_strings = <FILE>;
@@ -567,22 +565,13 @@ sub read_config{
         && $config_value{'BACKUP_NODE_SSH_PASS_MODE'} ne "nopass" ){
        printlog("ERROR", COMMON_MS0034, "BACKUP_NODE_SSH_PASS_MODE");
     }
-    if ($config_value{'PEER_NODE_SSH_PASS_MODE'} eq "passfile"){
-        if (!defined($config_value{'PEER_NODE_SSH_PASS_FILE'})){ 
-          printlog("ERROR", COMMON_MS0004, "PEER_NODE_SSH_PASS_FILE");
-        }
-        if ($config_value{'PEER_NODE_SSH_PASS_FILE'} !~ /^\//){
-          printlog("ERROR", COMMON_MS0005, "PEER_NODE_SSH_PASS_FILE");
-        }
+    if ($config_value{'PEER_NODE_SSH_PASS_MODE'} eq "passfile" && $config_value{'PEER_NODE_SSH_PASS_FILE'} !~ /^\//){
+        printlog("ERROR", COMMON_MS0005, "PEER_NODE_SSH_PASS_FILE");
     }
-    if ($config_value{'BACKUP_NODE_SSH_PASS_MODE'} eq "passfile"){
-        if (!defined($config_value{'BACKUP_NODE_SSH_PASS_FILE'})){
-          printlog("ERROR", COMMON_MS0004, "BACKUP_NODE_SSH_PASS_FILE");
-        }
-        if ($config_value{'BACKUP_NODE_SSH_PASS_FILE'} !~ /^\//){
-          printlog("ERROR", COMMON_MS0005, "BACKUP_NODE_SSH_PASS_FILE");
-        }
+    if ($config_value{'BACKUP_NODE_SSH_PASS_MODE'} eq "passfile" && $config_value{'BACKUP_NODE_SSH_PASS_FILE'} !~ /^\//){
+        printlog("ERROR", COMMON_MS0005, "BACKUP_NODE_SSH_PASS_FILE");
     }
+
     # VIP_SLAVE が disable の場合、VIP-SLAVE_ResourceID の値を空にする
     if ($config_value{'VIP_SLAVE'} eq "disable"){
         $config_value{'VIP-SLAVE_ResourceID'} = "";
@@ -658,7 +647,7 @@ sub read_hacf{
         # comment format : #<comment>
         $line =~ s/\#.*//g;
         # format of bindnetaddr in corosync.conf: bindnetaddr: <network address> or <IP adress>
-        if ($line =~ /^\s*bindnetaddr:\s+(\S+)(\s.*)?$/){
+        if ($line =~ /^\s*bindnetaddr:\s+(\S+)/){
             push(@iclan_netaddr, $1);
         }
     }
@@ -721,7 +710,6 @@ sub get_iclan_ipaddr{
     my $nic;
     my %iclan_ipaddr;
     my $command;
-    my @record;
 
     # 自身のノードと相手先のノードの IC-LAN IP アドレスを取得する
     for (my $count = 0; $count < 2; $count++){
@@ -730,26 +718,21 @@ sub get_iclan_ipaddr{
             # bindnetaddr にホストアドレスが設定されていると判断し、ifconfig
             # コマンドで確認した上で bindnetaddrの設定値を IC-LAN IP アドレス
             # として取得する
-            $command = "$IFCONFIG | $GREP -F \" $netaddr \"";
+            $command = "$IFCONFIG | $GREP \"$netaddr\"";
             if ($count == 0){
                 $results[0] = `$command`;
             }
             else {
                 @results = ssh_exec_command($ssh_info, $command);
             }
-            if ($results[0] =~ /inet\s(addr:)?([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)\s/) {
-                if ($2 eq $netaddr){
-                    if (!is_ipv4_format($2)){
-                        printlog("ERROR", COMMON_MS0052, $2);
-                    }
-                    push(@{$ipaddr_list{$site[$count]}}, $2);
-                    next;
-                }
+            if ($results[0] =~ /($netaddr)/){
+                push(@{$ipaddr_list{$site[$count]}}, $1);
+                next;
             }
 
             # bindnetaddr にネットワークアドレスが設定されている場合は、当該アドレスが
             # 設定されている NIC を特定する
-            $command = "$ROUTE";
+            $command = "$ROUTE | $GREP \"$netaddr\"";
             if ($count == 0){
                 $results[0] = `$command`;
             }
@@ -760,17 +743,8 @@ sub get_iclan_ipaddr{
             if (!$results[0]){
                 printlog("ERROR", COMMON_MS0039, $node_list[$count], $netaddr);
             }
-
-            @record = split (/\n/, $results[0]);
-            foreach (@record) {
-                # routeのコマンドの実行結果はスペース区切りとなるため、
-                # スペースをデリミタとしてsplitする
-                # $element[0]にIPアドレス、$element[7]にNIC名が格納される
-                my @element = split(/\s+/, $_);
-                if ($element[0] eq $netaddr) {
-                        $nic = $element[7];
-                }
-            }
+            @field = split(/\s+/, $results[0]);
+            $nic = $field[7];
 
             # 特定した NIC に設定されている IC-LAN IP アドレスを取得する
             $command = "$IFCONFIG \"$nic\"";
@@ -782,9 +756,6 @@ sub get_iclan_ipaddr{
             }
 
             if ($results[0] =~ /inet\s(addr:)?(\S+)\s/){
-                if (!is_ipv4_format($2)){
-                    printlog("ERROR", COMMON_MS0052, $2);
-                }
                 push(@{$ipaddr_list{$site[$count]}}, $2);
             }
             else {
@@ -838,6 +809,35 @@ sub ssh_exec_command {
     return @results;
 }
 
+
+sub rsync_exec_command {
+    my ($ssh_info, $remote, $local, $put_flag) = @_; 
+    my $ssh;
+
+    if ($ssh_info->pass() eq ""){
+        $ssh = Net::OpenSSH->new($ssh_info->address(), user => $ssh_info->user());
+    }
+    else{
+        $ssh = Net::OpenSSH->new($ssh_info->address(), user => $ssh_info->user(), password => $ssh_info->pass());
+    }
+    $ssh->error and printlog("ERROR", COMMON_MS0016);
+
+    my %opts = (
+        archive => 1,
+        verbose => 1,
+    );
+
+	if ($put_flag){
+		$ssh-> rsync_put(\%opts, $local, $remote) or $ssh->error and printlog("ERROR", COMMON_MS0017);
+	}
+	else {
+    	$ssh-> rsync_get(\%opts, $remote, $local) or $ssh->error and printlog("ERROR", COMMON_MS0017);
+    }
+
+    return 0;
+}
+
+
 sub scp_exec_command {
     my ($ssh_info, $remote, $local) = @_; 
     my $ssh;
@@ -866,8 +866,9 @@ sub get_xlog_filename{
     
     # XLOG 位置の情報をパース
     # xlog location format : <xlogid>/<xrecoff>
-    # <xlogid>と<xrecoff>は16進数
-    if ($xlog_location !~ /^([0-9A-F]+)\/([0-9A-F]+)$/){
+    # <xlogid>は1,2桁の16進数
+    # <xrecoff>は7.8桁の16進数
+    if ($xlog_location !~ /^([0-9A-F]+)\/([0-9A-F]{7,8})$/){
         printlog("ERROR", COMMON_MS0013);
     }
     $uxlog_id = $1;
@@ -892,13 +893,13 @@ sub compare_lsn{
     my $lsn2_left_field;
     my $lsn2_right_field;
 
-    if ($lsn1 !~ /^([0-9A-F]+)\/([0-9A-F]+)$/){
+    if ($lsn1 !~ /^([0-9A-F]+)\/([0-9A-F]{7,8}+)$/){
         printlog("ERROR", COMMON_MS0013);
     }
     $lsn1_left_field = hex($1);
     $lsn1_right_field = hex($2);
 
-    if ($lsn2 !~ /^([0-9A-F]+)\/([0-9A-F]+)$/){
+    if ($lsn2 !~ /^([0-9A-F]+)\/([0-9A-F]{7,8}+)$/){
         printlog("ERROR", COMMON_MS0013);
     }
     $lsn2_left_field = hex($1);
@@ -922,14 +923,35 @@ sub compare_lsn{
 sub get_controldata_value{
     my (@controldata_strings) = @_;
     my %controldata_value;
-    my @check_key_list = ("pg_control last modified", "Database system identifier", "Latest checkpoint's TimeLineID", "Latest checkpoint's REDO location", "Latest checkpoint's REDO WAL file");
+    my @check_key_list = ("update_time", "database", "time_line_id", "xlog_location");
 
-	my @key_val;
-
-	foreach my $line (@controldata_strings) {
-		@key_val = split(/:/, $line);
-		($controldata_value{$key_val[0]} = $key_val[1]) =~ s/^ *(.*?) *$/$1/;
-	}
+    foreach my $line (@controldata_strings){
+        # pg_control last modified format : pg_control last modified: <Wday> <Mon> <Mday> <Hour>:<Min>:<Sec> <Year>
+        if ($line =~ /^pg_control\s+last\s+modified:\s+(.+)$/){
+            $controldata_value{'update_time'} = $1;
+        }
+        # Database system identifier format : Database system identifier: <Database>
+        if ($line =~ /^Database\s+system\s+identifier:\s+([0-9A-F]+)$/){
+            $controldata_value{'database'} = $1;
+        }
+        # TimeLineID format : Latest checkpoint's TimeLineID: <TimeLineID>
+        # <TimeLineID>は10進数
+        if ($line =~ /^Latest\s+checkpoint's\s+TimeLineID:\s+([0-9A-F]+)$/){
+            $controldata_value{'time_line_id'} = $1;
+        }
+        # location format : Latest checkpoint's REDO location: <location>
+        if ($line =~ /^Latest\s+checkpoint's\s+REDO\s+location:\s+([0-9A-F\/]+)$/){
+            $controldata_value{'xlog_location'} = $1;
+        }
+        # checkpoint time format : Time of latest checkpoint: <Wday> <Mon> <Mday> <Hour>:<Min>:<Sec> <Year>
+        if ($line =~ /^Time\s+of\s+latest\s+checkpoint:\s+(.+)$/){
+            $controldata_value{'checkpoint_time'} = $1;
+        }
+        # WAL file name format : Latest checkpoint's REDO WAL file: <WAL>
+        if ($line =~ /^Latest\s+checkpoint's\s+REDO\s+WAL\s+file:\s+([0-9A-F]+)$/){
+            $controldata_value{'wal_filename'} = $1;
+        }
+    }
 
     foreach my $key (@check_key_list) {
         if (!exists($controldata_value{$key})){
@@ -938,7 +960,7 @@ sub get_controldata_value{
     }
 
 
-    return \%controldata_value;
+    return %controldata_value;
 }
 
 sub get_start_wal_filename {
@@ -955,9 +977,9 @@ sub get_recoverywal {
     my (@controldata_strings) = @_;
     my $wal = '';
 
-    my $controldata_value = (get_controldata_value(@controldata_strings));
+    my %controldata_value = get_controldata_value(@controldata_strings);
 
-    $wal = get_xlog_filename($controldata_value->{"Latest checkpoint's REDO location"}, $controldata_value->{"Latest checkpoint's TimeLineID"});
+    $wal = get_xlog_filename($controldata_value{'xlog_location'}, $controldata_value{'time_line_id'});
 
     return $wal;
 }
@@ -1024,11 +1046,12 @@ sub get_node{
 }
 
 
-sub get_pg_version_num {
+sub get_pg_version{
     my ($postgres_path, $exec_user) = @_;
     my $result;
+    my @results;
+    my @version_factor;
     my $pg_command_user = "postgres";
-    my $version_num;
 
     if (defined($exec_user) && $exec_user ne "root"){
         $result = exec_command("$postgres_path --version");
@@ -1036,38 +1059,24 @@ sub get_pg_version_num {
         $result = exec_command("$SU - $pg_command_user -c  \"$postgres_path --version\"");
     }
 
-    $result =~ /(\d+)(?:[^\.]+)?(?:\.(\d+)(?:[^\.]+)?)?(?:\.(\d+))?$/;
-    if ($1 >= 10){
-        # PostgreSQL 10以上の場合
-        # $1 : Major Version
-        # $2 : Minor Version
-        # $2が未定義の場合は、betaまたはrc版
-        if (!defined($2)){
-            $version_num = 10000 * $1;
-        }
-        else{
-            $version_num = 10000 * $1 + $2;
+    @results = split(/\s/,$result);
+    chomp $results[2];
+
+    @version_factor = split(/\./,$results[2]);
+
+    if (@version_factor == 2){
+        if ($version_factor[1] =~ /(^\d+)\D+/){
+            $version_factor[1] = $1;
+            $version_factor[2] = "0";
         }
     }
-    else{
-        # PostgreSQL 9.6以下の場合
-        # $1.$2 : Major Version
-        # $3    : Minor Version
-        # $3が未定義の場合は、betaまたはrc版
-        if (!defined($3)){
-            $version_num = 100 * (100 * $1 + $2);
-        }
-        else{
-            $version_num = 100 * (100 * $1 + $2) + $3;
-        }
-    }
-    return $version_num;
+
+    return sprintf("%s.%s.%s", $version_factor[0], $version_factor[1], $version_factor[2]);
 }
 
 sub check_support_version {
     my ($postgres_path, $exec_user) = @_;
     my @version_factor;
-    my $version_num;
 
     # Pacemaker がサポート対象バージョンであるかを確認する
     @version_factor = split(/\./, get_pm_version());
@@ -1076,14 +1085,10 @@ sub check_support_version {
     }
 
     # PostgreSQL がサポート対象バージョンであるかを確認する
-    $version_num = get_pg_version_num($postgres_path, $exec_user);
-    if (int($version_num / 10000) != 11){
-        if ($version_num >= 100000) {
-            printlog("ERROR", COMMON_MS0037, int($version_num / 10000));
-        } else {
-            printlog("ERROR", COMMON_MS0037, sprintf("%d.%d",
-                int($version_num / 10000), int(($version_num / 100) % 100)));
-        }
+    @version_factor = split(/\./, get_pg_version($postgres_path, $exec_user));
+    if ($version_factor[0] != 9 ||
+        ($version_factor[1] != 4 && $version_factor[1] != 5)){
+            printlog("ERROR", COMMON_MS0037, get_pg_version($postgres_path), $exec_user);
     }
 }
 
@@ -1107,54 +1112,45 @@ sub ifcheckd_running {
 
 sub get_pg_command_path {
     my ($pg_path, $exec_user) = @_;
-    my @command_list = ("postgres", "psql", "pg_controldata", "pg_basebackup", "pg_waldump", "pg_rewind", "pg_ctl");
-    my $pg_command_user = "postgres";
     my %command_path;
-    my $pg_config_path;
-    my $pgbin;
-    my @missing_commands = ();
-    my $exit_code;
+    my @command_list = ("postgres", "psql", "pg_controldata", "pg_basebackup", "vacuumdb", "pg_xlogdump");
+    my $pg_command_user = "postgres";
 
-    # PostgreSQL の bin ディレクトリのパスを pg_config から取得する
-    # PGPATH が指定されている場合は PGPATH 配下の pg_config を使用する
-    # PGPATH が指定されていない場合は PATH 経由の pg_config を使用する
-    if ($pg_path) {
-        $pg_config_path = File::Spec->catfile($pg_path, "pg_config");
-        $pgbin = `$pg_config_path --bindir 2> /dev/null`;
-    } else {
-        $pg_config_path = "pg_config";
-        if (!$exec_user || $exec_user eq "root") {
-            $pgbin = `$SU - $pg_command_user -c "$pg_config_path --bindir 2> /dev/null"`;
-        } else {
-            $pgbin = `$pg_config_path --bindir 2> /dev/null`;
+    if (!$pg_path){
+        $pg_path = "";
+    }
+    if ($pg_path && $pg_path =~ /\S+\/$/){
+        $pg_path =~ s/\/$//;
+    }
+
+    foreach my $command (@command_list){
+        if (!defined($exec_user) || $exec_user eq "root"){
+            $command_path{$command} = `$SU - $pg_command_user -c "$WHICH $command" 2> /dev/null`;
         }
-    }
-    $exit_code = $? >> 8;
-    if ($exit_code != 0) {
-        printlog("ERROR", COMMON_MS0011, $pg_config_path);
-    }
-    chomp $pgbin;
-
-    # PostgreSQLコマンドのコマンドパスを生成して返却する
-    # 存在しないコマンドがある場合は異常終了する
-    foreach (@command_list) {
-        $command_path{$_} = File::Spec->catfile($pgbin, $_);
-        `$WHICH $command_path{$_} 2> /dev/null`;
-        $exit_code = $? >> 8;
-        if ($exit_code != 0) {
-            push(@missing_commands, $_);
+        else{
+            $command_path{$command} = `$WHICH $command 2> /dev/null`;
         }
-    }
+        if (!$command_path{$command}){
+            $command_path{$command} = $pg_path."/".$command;
 
-    printlog("DEBUG", "PostgreSQL command path :\n[0]\n", join("\n", values %command_path));
-
-    if (@missing_commands) {
-        printlog("ERROR", COMMON_MS0036, $pgbin, join(',', @missing_commands));
+            # postgres の場合のみPGPATHが正しく設定されているかを確認
+            if ($command eq "postgres"){
+                `$WHICH $command_path{$command} 2> /dev/null`;
+                my $return = $? >> 8;
+                if ($return != 0){
+                    printlog("ERROR", COMMON_MS0036, $pg_path);
+                }
+            }   
+        }
+        else{
+            chomp $command_path{$command};
+        }
     }
 
     return %command_path;
 }
 
+
 sub get_pm_version{
     my $result;
     my @results;
@@ -1212,14 +1208,8 @@ sub printlog {
     }
 
     if ($log_level eq "ERROR"){
-        print $message;
-		exit(1);
+        die $message;
     }
-	elsif ($log_level eq "DEBUG"){
-		if ($PGRex::common::additional_information_mode) {
-			print $message;
-		}
-	}
     else{
         print $message;
     }
@@ -1263,272 +1253,15 @@ sub get_ssh_passwd {
         open (FILE, $ssh_pass_file) or printlog("ERROR", COMMON_MS0032, $ssh_pass_file);
         $passwd = <FILE>;
         close (FILE);
-        
-        if (!defined($passwd)){
-            printlog("ERROR", COMMON_MS0033, $ssh_pass_file);
-        }
-        
+
         chomp $passwd;
 
         if ( $passwd eq "" ){
-            printlog("ERROR", COMMON_MS0033, $ssh_pass_file);
+            printlog("ERROR", COMMON_MS0033);
         }
     }
 
     return $passwd;
 }
 
-
-sub check_dbcluster_access {
-    my ($cluster_path) = @_;
-    my %procs;
-    my $result = "";
-
-	stat $LSOF || printlog("ERROR", COMMON_MS0053, $LSOF, $!);
-
-    open my $in, '-|', "$LSOF +D $cluster_path -F cuLR0 2> /dev/null" || printlog("ERROR", COMMON_MS0046, $!);
-	# Suppress display of stderr only, because error of lsof command is unknown
-
-    while (<$in>) {
-        my $pid;
-
-        while (/([a-zA-Z])([^\x00]+)\x00/g) {
-            if ($1 eq "p") {
-                $pid = $2;
-            } else {
-                $procs{$pid}{$1} = $2;
-            }
-        }
-    }
-    close $in;
-
-    if (keys %procs) {
-        foreach my $key (keys %procs) {
-            # filter out forked children
-            next if (defined $procs{$procs{$key}{R}});
-
-            $result .= "PID: $key / COMMAND: $procs{$key}{c}, USER=$procs{$key}{L}($procs{$key}{u})\n";
-        }
-        printlog("ERROR", COMMON_MS0047, $result);
-    }
-}
-
-
-sub create_pid_file {
-    my $pid_fullname = PID_FILEDIR."/".PID_FILENAME;
-    my $pid;
-    my $exit_code;
-
-    for(my $ntries = 0; ; $ntries++) {
-
-        # create pid file
-        if (sysopen my $handle, "$pid_fullname", O_WRONLY|O_CREAT|O_EXCL) {
-            print $handle "$$";
-            close $handle;
-            return;
-        } elsif ($! != Errno::EEXIST) {
-            printlog("ERROR", COMMON_MS0048, PID_FILENAME, $!);
-        }
-
-		if ($ntries > 5) {
-			last;
-		}
-		$ntries != 0 && sleep 1;
-
-        # check pid file
-		open FILE, "< $pid_fullname" || continue;
-		$pid = <FILE>; # If it can not read, $pid is UNDEF
-		close FILE;
-
-		# other pg-rex_tool has already started
-		if (defined($pid)) {
-			chomp $pid;
-			if ($pid ne "") {
-				`$KILL -s 0 $pid 2> /dev/null`;
-				$exit_code = $? >> 8;
-				if ($exit_code == 0) {
-					printlog("ERROR", COMMON_MS0049, $pid);
-				}
-			}
-		}
-
-        # unlink pid file
-        unlink $pid_fullname;
-		if ($! == Errno::EISDIR) {
-			printlog("ERROR", COMMON_MS0051, $!);
-		}
-    }
-
-    printlog("ERROR", COMMON_MS0050, PID_FILENAME);
-}
-
-sub unlink_pid_file {
-    my $pid_fullname = PID_FILEDIR."/".PID_FILENAME;
-
-    if (!unlink $pid_fullname) {
-        printlog("LOG", COMMON_MS0051, PID_FILENAME, $!);
-    }
-}
-
-sub is_ipv4_format{
-    # 引数に指定されたIPアドレスがIPv4の形式であるかを確認する
-    # IPv4の形式の場合は、戻り値に「0」を返却する
-    # IPv4の形式でない場合は、戻り値に「1」を返却する
-    # チェック対象のIPアドレスは、コマンド実行結果から取得しているため、
-    # 厳密なIPv4の正規表現でIPアドレスであるかは確認しない
-    my ($ipaddr) = @_;
-    if ($ipaddr =~ /^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$/){
-        return 1; # OK
-    } else {
-        return 0; # NG
-    }
-}
-
-# ローカルとリモートのディレクトリの差分から、サイズ情報を含む同期対象ファイルを取得する
-# 返却値のsync_filesはkeyが同期対象ファイル名、valueがファイルサイズのハッシュ
-sub get_sync_files {
-	my ($ssh_info, $remote_dir, $local_dir, $local_flag) = @_;
-	my @local_ls = ();
-	my @remote_ls = ();
-	my @sync_filelist_to_remote = ();
-	my @sync_filelist_to_local = ();
-	my %cnt = ();
-	my %sync_files = ();
-
-	# ローカルのアーカイブ一覧取得
-	@local_ls = `$LS -go $local_dir | $AWK '{print \$3,\$7}'`;
-	chomp @local_ls;
-	shift(@local_ls);
-
-	# リモートのアーカイブ一覧取得
-	my $command = "$LS -go $remote_dir | $AWK '{print \$3,\$7}'";
-	my @results = ssh_exec_command($ssh_info, $command);
-	@remote_ls = split(/\n/, $results[0]);
-	shift(@remote_ls);
-
-	# ハッシュの初期化
-	map { $cnt{$_}-- } @remote_ls;
-
-	# ローカルにのみ存在するものおよびファイルサイズが異なるものを抽出
-	@sync_filelist_to_remote = grep { ++$cnt{$_} == 1 } @local_ls;
-
-	# ローカルを基準にする場合は、上記で抽出したリストから返却値用のハッシュを生成する
-	# ローカルを基準にしない場合は、リモートにのみ存在するものおよびファイルサイズが異なるものを抽出し、
-	# 返却値用のハッシュを生成する
-	if ($local_flag) {
-		foreach (@sync_filelist_to_remote){
-			if ($_ =~ /^\s*(\d+)\s(.+)$/ ) {
-				$sync_files{$2} = $1;
-			}
-		}
-	} else {
-		@sync_filelist_to_local = grep { $cnt{$_} == -1 } @remote_ls;
-		foreach (@sync_filelist_to_local){
-			if ($_ =~ /^\s*(\d+)\s(.+)$/ ) {
-				$sync_files{$2} = $1;
-			}
-		}
-	}
-	
-	return %sync_files;
-}
-
-# ローカルのアーカイブファイルをリモートへ送信する
-sub send_archive {
-	my ($ssh_info, $dir, $bytes_per_wal_segment, $sync_targets) = @_;
-	my $ssh;
-
-	if ($ssh_info->pass() eq ""){
-		$ssh = Net::OpenSSH->new($ssh_info->address(), user => $ssh_info->user());
-	} else {
-		$ssh = Net::OpenSSH->new($ssh_info->address(), user => $ssh_info->user(), password => $ssh_info->pass());
-	}
-	$ssh->error and printlog("ERROR", COMMON_MS0016);
-
-	foreach my $name (sort keys %$sync_targets){
-		my $size = $$sync_targets{$name};
-		if ((my $ret = check_archivefile_size($name, $size, $bytes_per_wal_segment)) > 0 ){
-			if ($ret == 1) {
-				# アーカイブWALのサイズが制御ファイルの"Bytes per WAL segment"と一致しない
-				printlog("ERROR", COMMON_MS0055, $name, $bytes_per_wal_segment);
-			}
-			if ($ret == 2) {
-				# アーカイブWAL以外のサイズが0
-				printlog("ERROR", COMMON_MS0054, $name);
-			}
-		}
-		open(my $LOCAL, "$TAR czf - -C $dir $name |");
-		my $REMOTE = $ssh->pipe_in("$TAR mzxvf - -C $dir");
-		while (my $line = <$LOCAL>) {
-			print($REMOTE $line);
-		}
-		close $LOCAL;
-		close $REMOTE;
-	}
-	undef $ssh;
-}
-
-# リモートのアーカイブファイルをローカルに受信する
-sub receive_archive {
-	my ($ssh_info, $dir, $bytes_per_wal_segment, $sync_targets) = @_;
-	my $ssh;
-
-	if ($ssh_info->pass() eq ""){
-		$ssh = Net::OpenSSH->new($ssh_info->address(), user => $ssh_info->user());
-	} else {
-		$ssh = Net::OpenSSH->new($ssh_info->address(), user => $ssh_info->user(), password => $ssh_info->pass());
-	}
-	$ssh->error and printlog("ERROR", COMMON_MS0016);
-
-	foreach my $name (sort keys %$sync_targets){
-		my $size = $$sync_targets{$name};
-		if ((my $ret = check_archivefile_size($name, $size, $bytes_per_wal_segment)) > 0 ){
-			if ($ret == 1) {
-				# アーカイブWALのサイズが制御ファイルの"Bytes per WAL segment"と一致しない
-				printlog("ERROR", COMMON_MS0055, $name, $bytes_per_wal_segment);
-			}
-			if ($ret == 2) {
-				# アーカイブWAL以外のサイズが0
-				printlog("ERROR", COMMON_MS0054, $name);
-			}
-		}
-		my $REMOTE = $ssh->pipe_out("$TAR czf - -C $dir $name");
-		open(my $LOCAL, "| $TAR mzxvf - -C $dir");
-		while (my $line = <$REMOTE>) {
-			print($LOCAL $line);
-		}
-		close $REMOTE;
-		close $LOCAL;
-	}
-	undef $ssh;
-}
-
-# アーカイブファイルのサイズの妥当性を確認する (正常であれば0を返却する)
-#  - アーカイブWALの場合、指定されたサイズと一致することを確認する
-#    一致しない場合は1を返却する
-#  - アーカイブWAL以外の場合、サイズが0でないことを確認する
-#    サイズが0の場合は2を返却する
-# 
-# ※  圧縮されたアーカイブWALはサイズが0でないことを確認する 
-sub check_archivefile_size {
-	my ($name, $size, $expected_size) = @_;
-
-	# アーカイブWALのサイズ確認
-	if ($name =~ /^[0-9A-F]{24}$/) {
-		if ($size != $expected_size){
-			return 1;
-		}
-	}
-
-	# アーカイブWAL以外のサイズ確認
-	if ($name =~ /^([0-9A-F]{24})(\.gz|\.bz2)$/ || 
-	    $name =~ /^(.*)(\.history|\.partial|\.backup)(\.gz|\.bz2)?$/) {
-		if ($size == 0){
-			return 2;
-		}
-	}
-
-	return 0;
-}
-
 1;
